/* tkinactive.c -- Tk "inactive" command example extension. */

/* $Id$ */

#include <tcl.h>
#include <tk.h>

#include <string.h>

/* Local copy of TkGetDisplayOf, which is not included as a stub, I
 * guess. */

int
LocalGetDisplayOf(interp, objc, objv, tkwinPtr)
    Tcl_Interp *interp;		/* Interpreter for error reporting. */
    int objc;			/* Number of arguments. */
    Tcl_Obj *CONST objv[];	/* Argument objects. If it is present,
				 * "-displayof" should be in objv[0] and
				 * objv[1] the name of a window. */
    Tk_Window *tkwinPtr;	/* On input, contains main window of
				 * application associated with interp.  On
				 * output, filled with window specified as
				 * option to "-displayof" argument, or
				 * unmodified if "-displayof" argument was not
				 * present. */
{
    char *string;
    int length;
    
    if (objc < 1) {
	return 0;
    }
    string = Tcl_GetStringFromObj(objv[0], &length);
    if ((length >= 2) &&
	    (strncmp(string, "-displayof", (unsigned) length) == 0)) {
        if (objc < 2) {
	    Tcl_SetStringObj(Tcl_GetObjResult(interp),
		    "value for \"-displayof\" missing", -1);
	    return -1;
	}
	string = Tcl_GetStringFromObj(objv[1], NULL);
	*tkwinPtr = Tk_NameToWindow(interp, string, *tkwinPtr);
	if (*tkwinPtr == NULL) {
	    return -1;
	}
	return 2;
    }
    return 0;
}


/*
 *-----------------------------------------------------------------------------
 *
 * TkInactiveCmd --
 *
 * 	Implements the tkinactive command, which is invoked like so:
 * 	tkinactive ?-displayof window? ?reset?.
 *
 * Results:
 *	None.
 *
 * Side Effects:
 *	None.
 *
 *-----------------------------------------------------------------------------
 */

static int
TkInactiveCmd(ClientData clientData, Tcl_Interp *interp,
	  int objc, Tcl_Obj *CONST objv[]) {

    int skip = 1;
    int offset = 1; /* Offset in objv for use in different commands */
    Tk_Window tkwin = NULL;

    tkwin = Tk_MainWindow(interp); /* Get the main window. */

    if (tkwin == NULL) {
	return TCL_ERROR;
    }

    skip = LocalGetDisplayOf(interp, objc - offset, objv + offset, &tkwin);
    if (skip < 0) {
	return TCL_ERROR;
    }
    if (objc - skip == offset) {
	long inactive;

	inactive = (Tcl_IsSafe(interp) ? -1 :
		    Tk_GetUserInactiveTime(Tk_Display(tkwin)));
	Tcl_SetObjResult(interp, Tcl_NewLongObj(inactive));
    } else if (objc - skip == 1 + offset) {
	char *string;
	string = Tcl_GetStringFromObj(objv[objc-1], NULL);
	if (strcmp(string, "reset") != 0) {
	    Tcl_Obj *msg = Tcl_NewStringObj("bad option \"", -1);
	    Tcl_AppendStringsToObj(msg, string,
				   "\": must be reset", NULL);
	    Tcl_SetObjResult(interp, msg);
	    return TCL_ERROR;
	}
	if (Tcl_IsSafe(interp)) {
	    Tcl_SetResult(interp,
			  "resetting the user inactivity timer "
			  "is not allowed in a safe interpreter",
			  TCL_STATIC);
	    return TCL_ERROR;
	}
	Tk_ResetUserInactiveTime(Tk_Display(tkwin));
	Tcl_ResetResult(interp);
    } else {
	Tcl_WrongNumArgs(interp, 0, objv,
			 "?-displayof window? ?reset?");
	return TCL_ERROR;
    }
    return TCL_OK;
}


/*
 *-----------------------------------------------------------------------------
 *
 * Tkinactive_Init --
 *
 *	Create the 'tkinactive' command.
 *
 * Results:
 *	None.
 *
 * Side Effects:
 *	None.
 *
 *-----------------------------------------------------------------------------
 */

int Tkinactive_Init(Tcl_Interp *interp) {

#ifdef USE_TCL_STUBS
    if (Tcl_InitStubs(interp, "8", 0) == NULL) {
	return TCL_ERROR;
    }
#endif
#ifdef USE_TK_STUBS
    if (Tk_InitStubs(interp, "8", 0) == NULL) {
	return TCL_ERROR;
    }
#endif

    /* Create the command. */
    Tcl_CreateObjCommand(interp, "tkinactive", TkInactiveCmd,
			 (ClientData *) NULL, (Tcl_CmdDeleteProc *) NULL);

    if ( Tcl_PkgProvide(interp,
			"tkinactive",
			"0.1") != TCL_OK ) {
        return TCL_ERROR;
    }
    return TCL_OK;
}
